﻿using System;
using System.Windows.Forms;
using Eneter.Messaging.MessagingSystems.MessagingSystemBase;
using Eneter.Messaging.MessagingSystems.SynchronousMessagingSystem;
using Eneter.Messaging.MessagingSystems.TcpMessagingSystem;
using Eneter.Messaging.Nodes.Broker;
using Eneter.Messaging.Nodes.Dispatcher;
using Eneter.Messaging.Diagnostic;
using System.IO;

namespace DesktopCalculator
{
    public partial class Form1 : Form
    {
        private TcpPolicyServer myPolicyServer;
        private IDuplexBrokerClient myBrokerClient;
        private IDuplexDispatcher myDispatcher;

        public Form1()
        {
            InitializeComponent();

            // Enabling detailed debug tracing to the file.
            //EneterTrace.DetailLevel = EneterTrace.EDetailLevel.Debug;
            //EneterTrace.TraceLog = new StreamWriter("d:/tracefile.txt");


            // Start the policy server.
            // Note: Silverlight requires the policy server to start the communication.
            //       The policy server tells the silverlight if the communication is permitted.
            myPolicyServer = new TcpPolicyServer();
            myPolicyServer.StartPolicyServer();


            // Local messaging connecting the Dispatcher and the Broker.
            IMessagingSystemFactory aLocalMessaging = new SynchronousMessagingSystemFactory();

            // Create the broker forwarding notifications to subscribed clients.
            IDuplexBrokerFactory aBrokerFactory = new DuplexBrokerFactory();
            IDuplexBroker aBroker = aBrokerFactory.CreateBroker();

            // Attach the input channel to the broker.
            // Note: The broker will receive via this channel messages from the Dispatcher.
            IDuplexInputChannel aBrokerInputChannel =
                aLocalMessaging.CreateDuplexInputChannel("MyBrokerChannelId");
            aBroker.AttachDuplexInputChannel(aBrokerInputChannel);

            // Create the dispatcher.
            // The dispatcher will receive TCP messages and also local messages and forward them
            // to the broker.
            // (I.e. broker can get messages from local messaging and from TCP at the same time.)
            IDuplexDispatcherFactory aDispatcherFactory = new DuplexDispatcherFactory(aLocalMessaging);
            myDispatcher = aDispatcherFactory.CreateDuplexDispatcher();

            // Add the broker channel to the dispatcher.
            myDispatcher.AddDuplexOutputChannel("MyBrokerChannelId");

            // Attach the input channel receiving messages from the broker client.
            // Note: The channel id is same as the broker client has.
            IDuplexInputChannel aDispatcherLocalInputChannel =
                aLocalMessaging.CreateDuplexInputChannel("MyLocalBrokerClientId");
            myDispatcher.AttachDuplexInputChannel(aDispatcherLocalInputChannel);


            // Create the broker client notifying calculated results to the broker.
            myBrokerClient = aBrokerFactory.CreateBrokerClient();

            // Attach the output channel to the broker.
            // Note: The broker client will use this channel to send messages to the dispatcher.
            IDuplexOutputChannel aBrokerClientOutputChannel =
                aLocalMessaging.CreateDuplexOutputChannel("MyLocalBrokerClientId");
            myBrokerClient.AttachDuplexOutputChannel(aBrokerClientOutputChannel);


            // TCP messaging for the communication with the Silverlight client.
            IMessagingSystemFactory aTcpMessaging = new TcpMessagingSystemFactory();

            // Note: Silverlight can communicate only on ports: 4502-4532
            IDuplexInputChannel aTcpInputChannel =
                aTcpMessaging.CreateDuplexInputChannel("tcp://127.0.0.1:4510");

            // Finaly attach the TCP input channel to the dispatcher and start listening
            // to Silverlight clients.
            myDispatcher.AttachDuplexInputChannel(aTcpInputChannel);
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            // Detach input channels from the dispatcher and stop the listening.
            myDispatcher.DetachDuplexInputChannel();

            // Stop the policy server.
            myPolicyServer.StopPolicyServer();
        }

        private void CalculateBtn_Click(object sender, EventArgs e)
        {
            // Calculate numbers.
            int aResult = int.Parse(Number1TextBox.Text) + int.Parse(Number2TextBox.Text);

            // Show the result in the text box.
            ResultTextBox.Text = aResult.ToString();

            // Notify the result to the broker.
            myBrokerClient.SendMessage("MyResultMessageId", ResultTextBox.Text);
        }
    }
}
