﻿using System.Windows;
using System.Windows.Controls;
using Eneter.Messaging.MessagingSystems.MessagingSystemBase;
using Eneter.Messaging.MessagingSystems.TcpMessagingSystem;
using Eneter.Messaging.Nodes.Broker;
using System;

namespace SilverlightObserver
{
    public partial class MainPage : UserControl
    {
        private IDuplexBrokerClient myBrokerClient;
        private bool myIsSubscribedFlag;

        public MainPage()
        {
            InitializeComponent();

            // Create the BrokerClient to subscribe and receive notifications.
            IDuplexBrokerFactory aBrokerFactory = new DuplexBrokerFactory();
            myBrokerClient = aBrokerFactory.CreateBrokerClient();

            // Handle received notifications.
            myBrokerClient.BrokerMessageReceived += OnBrokerMessageReceived;


            // TCP messaging for the communication with the desktop application.
            // Note: Set the timeout for opening the connection and for sending messages.
            IMessagingSystemFactory aTcpMessaging = new TcpMessagingSystemFactory(1000);

            // Create the TCP output channel and attach it to the BrokerClient.
            IDuplexOutputChannel aTcpOutputChannel =
                aTcpMessaging.CreateDuplexOutputChannel("tcp://127.0.0.1:4510");

            try
            {
                myBrokerClient.AttachDuplexOutputChannel(aTcpOutputChannel);
            }
            catch (Exception)
            {
            }
        }


        private void LayoutRoot_Unloaded(object sender, RoutedEventArgs e)
        {
            // Detach the TCP output channel and stop the communication.
            myBrokerClient.DetachDuplexOutputChannel();
        }

        private void SubscribeBtn_Click(object sender, RoutedEventArgs e)
        {
            // If not subscribed, then subscribe for the same type
            // as is notified from the desktop.
            if (!myIsSubscribedFlag)
            {
                myBrokerClient.Subscribe("MyResultMessageId");
                myIsSubscribedFlag = true;
            }
        }

        private void UnsubscribeBtn_Click(object sender, RoutedEventArgs e)
        {
            // If subscribed, then unsubscribe.
            if (myIsSubscribedFlag)
            {
                myBrokerClient.Unsubscribe();
                myIsSubscribedFlag = false;
            }
        }

        private void OnBrokerMessageReceived(object sender, BrokerMessageReceivedEventArgs e)
        {
            // Display the notified message.
            ResultTextBox.Text = e.Message as string;
        }
    }
}
